# -*- coding: utf-8 -*-
##  Generated by PyVows v2.0.0  (2013/05/25)
##  http://pyvows.org

##  IMPORTS  ##
##
##  Standard Library
#
##  Third Party
#
##  PyVows Testing
from pyvows import Vows, expect

##  Local Imports
#import


##  TESTS  ##
@Vows.batch
class Utils(Vows.Context):

    def topic(self):
        return # return what you're going to test here

    ##  Now, write some vows for your topic! :)
    def should_do_something(self, topic):
        expect(topic)# <pyvows assertion here>

